package MyNotify;


#################################################
# Extract information from PAML programs runs   #
#           ------------------                  #
#   Written by Emanuel Maldonado - Jan 2015     #
#############################################################################
#  LMAP: Lightweight Multigene Analyses in PAML
#  MyNotify.pm module is part of LMAP package
#  Copyright (C) 2015  Emanuel Maldonado
#
#  MyNotify.pm is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MyNotify.pm is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with MyNotify.pm.  If not, see <http://www.gnu.org/licenses/>.
#############################################################################

use strict;
use warnings;

use Data::Dumper;

use Email::MIME;
use IO::All;
use Email::Sender;
use Email::Sender::Simple qw(sendmail);
use Email::Sender::Transport::SMTP qw();

use Try::Tiny;
use MyUtil qw(:NOTIFY :CONST);

BEGIN {
    require Exporter;

    our $VERSION = "1.0.0 Aug 14th, 2015";

    our @ISA = qw(Exporter);

    our @EXPORT = qw(&nnew &isValidAddr &isValidAddress &send &getEmailTo);

    our %EXPORT_TAGS = (
	INST => [qw(&isValidAddress)]
	);
};


our $MPID = "";
our $SDATE = "";
our $EMAILTO = "";
our $ATTCSVFL = "";
our $ATTCSVPTH = "";
our $ENABLEDLOG = "";

our $_PFFL = "";
our @_BODY = ();
our $_SMTP_SERVER = "";
our $_SMTP_PORT = "";
our $_SMTP_HELO = "";
our $_SMTP_SSL = 0;
our $_SMTPAUTH_USER = "";
our $_SMTPAUTH_PASSWD = "";
our $_FROM = "MMAP Program \<mmap\@lmap.biosoftware.net\>";
our $_SUBJECT = "Notification: MMAP $MPID Jobs Terminated!";
our $_CONTENT_TYPE = "text/plain; charset=utf-8";
our %_transpTBL = (
    host => $_SMTP_SERVER,
    port => $_SMTP_PORT,
    helo => $_SMTP_HELO,
    ssl => $_SMTP_SSL,
    sasl_username => $_SMTPAUTH_USER,
    sasl_password => $_SMTPAUTH_PASSWD,
    );

sub nnew {
    $EMAILTO = shift;
    $ATTCSVPTH = shift;
    $ATTCSVFL = shift;
    $MPID = shift;
    $SDATE = shift;
    $ENABLEDLOG = shift;
    $_PFFL = getPFL();    
    _readEP();
    _prepareBody();
}

sub getEmailTo() {
    return $EMAILTO;
}

sub isValidAddr {
    my $addr = shift;
    return 1 if(!$addr);

    if($addr =~ m/^[_a-zA-Z0-9][_a-zA-Z0-9\.-]*\@[a-zA-Z\.-]+(\.[a-zA-Z]{2,4})$/) {
	return 1;
    }
    return 0;
}

sub isValidAddress {
    my $addr = shift;
    if($addr =~ m/^[_a-zA-Z0-9][_a-zA-Z0-9\.-]*\@[a-zA-Z\.-]+(\.[a-zA-Z]{2,4})$/) {
	return 1;
    }
    return 0;
}


sub _readEP {
    my @PREF = split(/\n/,readPFL());
    my %EPFL = getPTBL();
    my $fxaddr;
    foreach (@PREF) {
	if(!$_SMTP_SERVER && $_ =~ m/^$EPFL{h}/) {
	    $_SMTP_SERVER = (split("=",$_,2))[1];
	    $_SMTP_SERVER =~ s/\<no\>\s*//;
	    chomp $_SMTP_SERVER;
	}
	if(!$_SMTP_PORT && $_ =~ m/^$EPFL{p}/) {
	    $_SMTP_PORT = (split("=",$_,2))[1];
	    $_SMTP_PORT =~ s/^\<no\>\s*//;
	    chomp $_SMTP_PORT;
	}
	if(!$_SMTP_HELO && $_ =~ m/^$EPFL{e}/) {
	    $_SMTP_HELO = (split("=",$_,2))[1];
	    $_SMTP_HELO =~ s/^\<no\>\s*//;
	    chomp $_SMTP_HELO;
	}
	if(!$_SMTP_SSL && $_ =~ m/^$EPFL{s}/) {
	    $_SMTP_SSL = (split("=",$_,2))[1];
	    $_SMTP_SSL =~ s/^\<no\>\s*/0/;
	    $_SMTP_SSL =~ s/^\<yes\>\s*/1/;
	    chomp $_SMTP_SSL;
	}
	if(!$_SMTPAUTH_USER && $_ =~ m/^$EPFL{u}/) {
	    $_SMTPAUTH_USER = (split("=",$_,2))[1];
	    $_SMTPAUTH_USER =~ s/^\<no\>\s*//;
	    chomp $_SMTPAUTH_USER;
	}
	if(!$_SMTPAUTH_PASSWD && $_ =~ m/^$EPFL{w}/) {
	    $_SMTPAUTH_PASSWD = (split("=",$_,2))[1];
	    $_SMTPAUTH_PASSWD =~ s/^\<no\>\s*//;
	    chomp $_SMTPAUTH_PASSWD;
	}
	if($_ =~ m/^$EPFL{f}/) {
	    $fxaddr = (split("=",$_,2))[1];
	    $fxaddr =~ s/\<no\>\s*//;
	    chomp $fxaddr;
	    if($fxaddr) {
		if(isValidAddr($fxaddr)) {
		    $EMAILTO = $fxaddr if(!$EMAILTO); #set from pref file (default) if not given by user
		}
	    }
	}
    }
}

sub _prepareBody {  
    @_BODY = <<END_BODY;
Dear user <ADDRESS>,

This email is a notification from the LMAP application mmap.pl running your PAML jobs.

Your codeml tasks with ID:<MPID> running since <STARTDATE> have all finished with success!
<LOGLINE>

Thanks for using our software!

Don\'t forget to cite our publication:
<CITATION>

Best regards,

    LMAP authors.

LMAP Home Page:
<AVAILAPPURL>


END_BODY

my $logline = "Attached is the output log from all the runs, specifying which are correctly finished and which are not.";

    my $CITATION = CITATION;
    my $AVAILAPPURL = PROJWEBSITE;

    foreach (@_BODY) {
	$_ =~ s/<ADDRESS>/$EMAILTO/;
	$_ =~ s/<MPID>/$MPID/;
	$_ =~ s/<STARTDATE>/$SDATE/;
	$_ =~ s/<LOGLINE>/$logline/ if($ENABLEDLOG);
	$_ =~ s/<LOGLINE>\n?// if(!$ENABLEDLOG);
	$_ =~ s/<CITATION>/$CITATION/;
	$_ =~ s/<AVAILAPPURL>/$AVAILAPPURL/;
    }
    return @_BODY;
}

sub send {
    if(!$EMAILTO) {
	die "ERR::MyNotify:: Could not send email notification: no email given, neither specified in preferences file!\n";
    }
    if(!$_SMTP_SERVER) {
	die "ERR::MyNotify:: Could not send email notification: email settings not found in preferences file!\n";
    }

    my @parts = ( #attachs
		  Email::MIME->create( #attach body
				       attributes => {
					   content_type => $_CONTENT_TYPE,
					   encoding => "quoted-printable",
					   charset => "UTF-8",
				       },
				       body_str => join("",@_BODY),
		  )
	);

    push @parts, (Email::MIME->create( #attach CSV file from runs
					attributes => {
					    content_type => $_CONTENT_TYPE,
					    name => $ATTCSVFL,
					    filename => $ATTCSVFL,
					    encoding => "quoted-printable",
					    disposition  => "attachment",
					    name => $ATTCSVPTH.$ATTCSVFL,
					},
					body => io($ATTCSVPTH.$ATTCSVFL)->all, #load file
		       )) if($ENABLEDLOG);
    
    my $msg = Email::MIME->create(
	header_str => [
	    Sender => $_FROM,
	    From => "MMAP Program\<".$_SMTPAUTH_USER."\>",
	    To => $EMAILTO,
	    Subject => $_SUBJECT
	],
	parts => [ @parts ],
	);

    _updateSMTPTBL();

    try {
	sendmail($msg,{	transport => Email::Sender::Transport::SMTP->new(%_transpTBL) });
	print "\tMyNotify:: Email Notification Sent to $EMAILTO!\n";
    } catch {
	die "ERR::MyNotify:: Notification failed to address ($EMAILTO): Please run configure.pl: $_\n";
    };

}

sub _updateSMTPTBL {
    
    %_transpTBL = (
	host => $_SMTP_SERVER,
	port => $_SMTP_PORT,
	helo => $_SMTP_HELO,
	ssl => $_SMTP_SSL,
	sasl_username => $_SMTPAUTH_USER,
	sasl_password => $_SMTPAUTH_PASSWD,
    );
   
    foreach (keys %_transpTBL) {
	delete $_transpTBL{$_} if (!$_transpTBL{$_});
    }
    
    die "ERR::MyNotify:: Could not send notification: Please run configure.pl to configure how to interact with your SMTP server." if(!%_transpTBL);
}

END {}

1;



