#!/usr/bin/perl -w
# replace this line with "#!/usr/bin/env perl -w" in MACOS systems

#################################################
# Configuration script for LMAP package         #
#           ------------------                  #
#   Written by Emanuel Maldonado - Feb 2015     #
#############################################################################
#  Copyright (C) 2015  Emanuel Maldonado

#  configure.pl is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  configure.pl is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with configure.pl.  If not, see <http://www.gnu.org/licenses/>.
#############################################################################

use strict;
use Getopt::Long;
use Data::Dumper;
use File::Copy;
use File::Copy::Recursive qw(dircopy);
use File::Spec::Functions;
use FindBin;
use lib "$FindBin::Bin/LMAPlib/";
use MyNotify qw(:INST);
use MyUtil qw(:INST :CONST);
use Term::ReadKey;
use constant {
    VERSION => "1.0.0 Aug 14th, 2015",
    YESTAG => "<yes>",
    NOTAG => "<no>",
    DFLTEHLO => "lmap.biosoftware.net",
};

printLicense();
print "\nVERSION: ",VERSION,"\n\n";

my $makedefaultprefscoded;
my $help;

GetOptions(
    "preppfl!" => \$makedefaultprefscoded
    ) or "";


my $USRLBIN = "/usr/local/bin/"; #both linux and mac

if(defined $makedefaultprefscoded) { # only to update number of preferences
    my $TPFL;
    {
	local $/ = '%';
	<DATA>;
	$TPFL = <DATA>;
	close DATA;
    }
    open(PFL, ">", catfile($LMAPlibloc,"lmapp.enc")) or die("Could not create default preferences file: $!");
    print PFL getonecd("c",$TPFL);
    close PFL;
    print "\nDone!\n";
    exit 0;
}

my $DFL = "";
my %PREFTBL = ();
my @PREFTBLKeys = getPKeys();
my %PTBL = getPTBL();
    
foreach (@PREFTBLKeys) { #load defaults
    $PREFTBL{$_} = NOTAG;
}
$PREFTBL{$PTBL{e}} = DFLTEHLO;

print "\n:: Starting with general configurations...\n";

my $i = 1;
my $ans = 0;
do {
    print "\n$i) Would you like to disable the use of colors in scripts interaction with user? Type [Y]es/[N]o: ";
    $ans = <STDIN>;
    $ans =~ s/\s+//g;
} while ($ans !~ m/^[yYnN]/);

$PREFTBL{$PTBL{c}} = YESTAG if($ans =~ m/^[nN]/);

$i++;
$ans = 0;
my $msg = "";
$msg = "[Location not found!]" if(!-d $HOMEBINPATH);
print "\n$i) PAML binaries are expected to be located at: \n";
print "\ta) [User installation] $HOMEBINPATH $msg \n";
$msg =""; $msg = "[Location not found!]" if(!-d $USRLBIN);
print "\tb) [Repository installation] $USRLBIN $msg \n";
print "\tc) other ";
do {
    do {
	print "\nPlease select where to find PAML executables (your preferred location) or configure a new one.\nType [a/b] or enter the full path to the new directory > ";
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
    } while ($ans !~ m:(^\s*[aAbB]\s*$): && $ans !~ m:^(\/.+)+\/?$:);
    
    $PREFTBL{$PTBL{b}} = $HOMEBINPATH if($ans =~ m/^\s*[aA]\s*$/); #new absolute path
    $PREFTBL{$PTBL{b}} = $USRLBIN if($ans =~ m/^\s*[bB]\s*$/); #new absolute path
    $PREFTBL{$PTBL{b}} = $ans if($ans =~ m/^(\/.+)+\/?$/); #new absolute path
    print "Inserted location not valid!" if(!-d $PREFTBL{$PTBL{b}});
} while (!-d $PREFTBL{$PTBL{b}});

print "\n:: Starting email notification configurations...\n";

$i++;
$ans = 0;
do {
    print "\n$i) For these following steps, you will require additional information like (SMTP server hostname,\nport number, if connection is secure (SSL), the credentials (user and password) from\nan account that can be used to freely send email notifications from the LMAP applications.\n\nWould you like to proceed and setup email notifications? Type [Y]es or [N]o: ";
    $ans = <STDIN>;
    $ans =~ s/\s+//g;
} while ($ans !~ m:^[yYnN]:);

if($ans =~ m/^[yY]/) {

    $i++;
    $ans = 0;
    do {
	print "\n$i) Please, insert an SMTP mail server hostname and the required port number (the latter is optional;by default is automatically found) for sending emails (something like smtp.server.univ.fac.edu:25): ";
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
    } while ($ans !~ m/^(\w+\.)+\w+(:\d{1,5})?$/);

    if(index($ans,":") == -1) {
	$PREFTBL{$PTBL{h}} = $ans;
    } else {
	$PREFTBL{$PTBL{h}} = (split(':',$ans,2))[0];
	$PREFTBL{$PTBL{p}} = (split(':',$ans,2))[1];
    }
    
    $i++;
    $ans = 0;
    print "\n$i) The default value for greeting the SMTP server (HELO) is $PREFTBL{SMTPHELO}.";
    do {
	print "\nType [Y]es to accept or enter 'N:' followed by a new identification (your domain): ";
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
    } while ($ans !~ m/^[yY]|N:[\.\w]+/);   
    $PREFTBL{$PTBL{e}} = (split(":",$ans,2))[1] if($ans =~ m/^[nN]:/); 

    $i++;
    $ans = "";
    print "\n$i) Does your SMTP server require a secure or encrypted connection?\n";
    do {
	print "Please, " if($ans ne "");
	print "Type [Y]es or [N]o: ";
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
    } while ($ans !~ m:^[yYnN]:);   
    $PREFTBL{$PTBL{s}} = YESTAG if($ans =~ m/[yY]/);

    $i++;
    $ans = 0;
    print "\n$i) Please enter username and password for the SMTP account (quiet mode).";
    do {
	print "\nUsername (email adress): ";
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
	if(!isValidAddress($ans)) {
	    print "Your address was not found to be correct, please re-enter again.";
	}
    } while (!isValidAddress($ans));
    $ans =~ s/\@/\@/;
    $PREFTBL{$PTBL{u}} = $ans;
    $ans = "";
    my $ans1 = "";
    do {
	ReadMode 2;
	print "Password is not confirmed. Please, re-enter again: " if($ans ne $ans1);
	print "\nPassword: ";
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
	print "\nConfirm Password: ";
	$ans1 = <STDIN>;
	$ans1 =~ s/\s+//g;
	ReadMode 0;
    } while ($ans ne $ans1);
    $PREFTBL{$PTBL{w}} = $ans1;
    print "\nPassword is confirmed.\n";

    $i++;
    $ans = "";
    $ans1 = "";
    print "\n$i) Would you like to define a default email address to which notifications can be sent?";
    do {
	print "\nType [N]o or 'Y:' followed by your email address: " if(!$ans);
	$ans = <STDIN>;
	$ans =~ s/\s+//g;
	if($ans =~ m/^[yY]:/) {
	    $ans1 = (split(":",$ans,2))[1];
	    if(!isValidAddress($ans1)) {
		print "You address was not found to be correct, please re-enter again.";
		$ans = "";
	    }
	}
    } while ($ans !~ m/^([nN])|([yY]:)/);
    if($ans1) {
	$ans1 =~ s/\@/\@/;
	$PREFTBL{$PTBL{f}} = $ans1;
    }
}


print "\n:: Generating configuration file...\n";

{
    local $/ = '%';
    $DFL = getonecd("d",<DATA>);
}
close DATA;

my $v;
my $def = NOTAG;
foreach my $k (@PREFTBLKeys) {
    $v = $PREFTBL{$k};
    $DFL =~ s/$k=$def/$k=$v/;
}

open(PFL, ">", getPFL()) or die("Could not generate the preferences file: $!");
print PFL getcd($DFL);
close PFL;

print "\n:z: Setting up LMAP location...\n";

$i++;
$ans = "";
$msg = "";
my $dest = "";
my $c = "c";
do {
    print "\n$i) Where would you like to install LMAP applications?\n";
    $msg =""; $msg = "[Location not found!]" if(!-d $HOMEBINPATH);
    print "\ta) [User installation] $HOMEBINPATH $msg \n";
    $msg =""; $msg = "[Location not found!]" if(!-d $USRLBIN);
    print "\tb) [Repository installation] $USRLBIN $msg \n";
    $msg =""; $msg = "[Location not found!]" if(!-d $PREFTBL{$PTBL{b}});
    print "\tc) [Selected location] $PREFTBL{$PTBL{b}} $msg \n" and $c = 'd' if($PREFTBL{$PTBL{b}} ne $HOMEBINPATH && $PREFTBL{$PTBL{b}} ne $USRLBIN);
    print "\t$c) Do nothing, I will copy LMAP applications (and LMAPlib) to another directory. \n";
    print "Please, select an option. (Type [a/b/c/d]) > " if($c eq 'd');
    print "Please, select an option. (Type [a/b/c]) > " if($c eq 'c');
    $ans = <STDIN>;
    $ans =~ s/\s*//g;
} while ($ans !~ m:(^[aAbBcCdD]$):);    

if($ans =~ m/[aA]/) {
    $dest = $HOMEBINPATH;
} elsif ($ans =~ m/[bB]/) {
    $dest = $USRLBIN;
} elsif ($ans =~ m/[cC]/ && $c eq 'd') {
    $dest = $PREFTBL{$PTBL{b}};
} else {
    print ":z: You can now copy all LMAP applications and LMAPlib/ directory into your preferred binaries directory $HOMEBINPATH.\n";
}

if($dest) {
    foreach (glob("*p.pl")) {
	copy($_,$dest);
	chmod 0755,catfile($dest, $_); 
    }
    $File::Copy::Recursive::CPRFComp = 1;
    dircopy($LMAPlibloc,$dest);
}

print "\n:z: Installation concluded with success!!\n";
print ":: You may rerun this program anytime to change any required values.\n";


exit 0;








__DATA__
I0xNQVAgUHJlZmVyZW5jZXMKCiMjIyMjIyMjIyMjIz1HRU5FUkFMIFNFVFRJTkdTCiMjIyMjIyMj
IyMjIyMjIyMjIyMjIyMjIyMjIyMjIwoKI2J5IGRlZmF1bHQgUEFNTCBCSU4gUEFUSCBpcyBhdCB0
aGUgYmluIGRpcmVjdG9yeSBpbiB1c2VyIEhPTUUKT1RIRVJQQU1MQklOUEFUSD08bm8+CgojY2hv
b3NlIHRvIG5vdCB1c2UgQ09MT1Igd2l0aCAnPG5vPicgdmFsdWUKVVNFVEVSTUNPTE9SPTxubz4K
CiMjIyMjIyMjIyMjIz1FTUFJTCBOT1RJRklDQVRJT04gU0VUVElOR1MKIyMjIyMjIyMjIyMjIyMj
IyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMKCiNpZiBFTUFJTCBmdW5jdGlvbmFsaXR5IGlzIHRv
IGJlIHVzZWQsIHRoZW4gY2hhbmdlIHRoZSBmb2xsb3dpbmcgc2V0dGluZ3M6Cgojc2V0IHRoZSBh
ZGRyZXNzIGZvciB0aGUgU01UUCBzZXJ2ZXIgZS5nLiBzbXRwLnVuaXYuZmFjLmRlcGFydC5wdCBz
bXRwLmFsdW5vcy5kY2MuZmMudXAucHQKU01UUEhPU1ROQU1FPTxubz4KCiNpZGVudGl0eQpTTVRQ
SEVMTz08bm8+Cgojc2V0IHRoZSBwb3J0IG51bWJlciwgaW4gY2FzZSBkaWZmZXJlbnQgZnJvbSBk
ZWZhdWx0CiNub3QtU1NMIHBvcnQgZWcuIDI1IG9yIDU4NyBvdGhlcndpc2UgZWcuIDQ2NSAKU01U
UFBPUlQ9PG5vPgoKI3NldCB3aXRoIDx5ZXM+LCBpZiBTU0wgb3IgVFRMUyBjb25uZWN0aW9ucyBh
cmUgcmVxdWlyZWQsIG90aGVyd2lzZSBubyB3aXRoIDxubz4KU01UUFNTTD08bm8+CgojaWYgPHll
cz4gaW4gU01UUEFVVEgsIHRoZW4gcGxhY2UgdXNlcm5hbWUgKGVtYWlsIGFkZHJlc3MpIGFuZCBw
YXNzd29yZAojaWYgaXQgaXMgdGhlIGNhc2UsIGl0IHNob3VsZCBiZSBhbiBhY2NvdW50IHRoYXQg
b3RoZXIgcGVvcGxlIGNhbiBhbHNvIHVzZSwKI3RvIGF1dGhlbnRpY2F0ZSBhbmQgdGh1cyBiZSBh
YmxlIHRvIHNlbmQgbm90aWZpY2F0aW9ucyB0byB0aGVtc2VsdmVzLi4uClNNVFBBVVRIX1VTRVI9
PG5vPgpTTVRQQVVUSF9QQVNTV0Q9PG5vPgoKI3NldCBhIHVuaXF1ZSBTZW5kIFRvIGVtYWlsIGFk
ZHJlc3MsIGluIGNhc2Ugbm8gb25lIGVsc2UgaXMgdXNpbmcgdGhpcyBpbnN0YWxsYXRpb24KI3Ro
aXMgbWVhbnMgdGhhdCBpdCB3aWxsIGJlIHVzZWQgYnkgZGVmYXVsdCwgaWYgbm90IGdpdmVuIHdp
dGggLWUgb3B0aW9uIGluIE1QQU1MLgpGSVhFRE5PVElGWVRPPTxubz4K
%
#LMAP Preferences

############=GENERAL SETTINGS
##############################

#by default PAML BIN PATH is at the bin directory in user HOME
OTHERPAMLBINPATH=<no>

#choose to not use COLOR with '<no>' value
USETERMCOLOR=<no>

############=EMAIL NOTIFICATION SETTINGS
#########################################

#if EMAIL functionality is to be used, then change the following settings:

#set the address for the SMTP server e.g. smtp.univ.fac.depart.pt
SMTPHOSTNAME=<no>

#identity
SMTPHELO=<no>

#set the port number, in case different from default
#not-SSL port eg. 25 or 587 otherwise eg. 465 
SMTPPORT=<no>

#set with <yes>, if SSL or TTLS connections are required, otherwise no with <no>
SMTPSSL=<no>

#if <yes> in SMTPAUTH, then place username (email address) and password
#if it is the case, it should be an account that other people can also use,
#to authenticate and thus be able to send notifications to themselves...
SMTPAUTH_USER=<no>
SMTPAUTH_PASSWD=<no>

#set a unique Send To email address, in case no one else is using this installation
#this means that it will be used by default, if not given with -e option in MPAML.
FIXEDNOTIFYTO=<no>
