#!/usr/bin/perl -w
# replace this line with "#!/usr/bin/env perl -w" in MACOS systems

#################################################
# Installation script for LMAP requirements.    #
#           ------------------                  #
#   Written by Emanuel Maldonado - Feb 2015     #
#############################################################################
#  Copyright (C) 2015  Emanuel Maldonado

#  install.pl is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  install.pl is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with install.pl.  If not, see <http://www.gnu.org/licenses/>.
#############################################################################

use strict;
use Getopt::Long;
use Data::Dumper;
use CPAN;
use CPAN::Shell;

use constant {
    VERSION => '1.0.0 Aug 14th, 2015'
};

printLicense();
print "\nVERSION: ",VERSION,"\n\n";


if($> != 0) { #requires root admin priv : $EFFECTIVE_USER_ID root = id 0
    print "Cannot proceed. Please, re-run with sudo: sudo ./install.pl\n";
    exit;
}
    
print ":z: Installing required Perl modules...\n";
my @iModules = (
    "Email::MIME",
    "IO::All",
    "Email::Sender",
    "Sys::Info",
    "Term::ReadKey",
    "Thread::Semaphore",
    "Statistics::Distributions",
    "Math::Cephes",
    "Bio::TreeIO",
    "File::Copy",
    "File::Copy::Recursive"
    );

foreach (@iModules) {
    if(!CPAN->has_inst($_)) {
	print "Installing $_ ...\n";
	CPAN::Shell->install($_);
    }
}

if($^O eq "linux") {
    my $k = qx("apt-get");
    if($k =~ m/command\s+not\s+found/) {
	print "Failed to execute package manager apt-get: not available in your system.";
	print "Please try to install manually, using a different program.";
	print "Software required: paml, screen and sendmail-bin\n";
	exit 1;
    }
    print ":: All Perl modules successfully installed...\n";
    
    print "\n:z: Installing other dependencies from repositories...\n";
    
    print ":: Trying to install PAML, screen and sendmail...\n";
    print `apt-get -y install paml screen sendmail-bin 2>&1`;
    print ":: Aplications successfully installed!\n";
}


exit 0;

sub printLicense {
    my @License = <<LIC;
LMAP package (install.pl, configure.pl, lmap.pl, gmap.pl, cmap.pl, mmap.pl, imap.pl, omap.pl, MyUtil.pm, MyNotify.pm, MyPhylo.pm, MyPAMLInfo.pm)
Copyright (C) 2015 Emanuel Maldonado
This program comes with ABSOLUTELY NO WARRANTY; for details see `LICENSE_GPL.txt' file.
This is free software, and you are welcome to redistribute it
under certain conditions; see `LICENSE_GPL.txt' file for details.

LIC

print @License;
    return;
}
